/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.demo.annotation;


import android.view.View;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.demo.annotation.databinding.AnnotationPdfSampleActivityBinding;
import com.compdfkit.tools.common.utils.animation.CFillScreenManager;
import com.compdfkit.tools.common.views.pdfproperties.CAnnotationType;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;

public class CAnnotSampleScreenManager {

    private AnnotationPdfSampleActivityBinding binding;

    private CFillScreenManager fillScreenManager = new CFillScreenManager();

    public void bind(AnnotationPdfSampleActivityBinding mainBinding) {
        this.binding = mainBinding;
        fillScreenManager.bindTopToolView(binding.flTool);
        fillScreenManager.bindBottomToolViewList(binding.annotationToolBar);
//        bind right view
        fillScreenManager.bindRightToolViewList(binding.pdfView.slideBar);
    }


    public void changeWindowStatus(CPreviewMode mode) {
        if (mode == CPreviewMode.Viewer) {
            fillScreenManager.removeAndHideToolView(binding.annotationToolBar);
        } else if (mode == CPreviewMode.Annotation) {
            fillScreenManager.bindBottomToolViewList(binding.annotationToolBar);
            fillScreenManager.showFromBottom(binding.annotationToolBar, 200);
        }
    }

    public void changeWindowStatus(CAnnotationType type) {
        if (type == CAnnotationType.INK) {
            fillScreenManager.hideFromTop(binding.pdfToolBar, 200);
            fillScreenManager.hideFromBottom(binding.annotationToolBar, 200);
            fillScreenManager.bindTopToolView(binding.inkCtrlView);
            binding.inkCtrlView.setVisibility(View.VISIBLE);
        } else {
            binding.inkCtrlView.setVisibility(View.GONE);
            fillScreenManager.removeToolView(binding.inkCtrlView);
            fillScreenManager.showFromTop(binding.pdfToolBar, 200);
            fillScreenManager.showFromBottom(binding.annotationToolBar, 200);
        }
    }

    public void changeWindowStatus(CPDFAnnotation.Type pdfType) {
        if (pdfType == CPDFAnnotation.Type.INK) {
            fillScreenManager.hideFromTop(binding.pdfToolBar, 200);
            fillScreenManager.hideFromBottom(binding.annotationToolBar, 200);
            fillScreenManager.bindTopToolView(binding.inkCtrlView);
            binding.inkCtrlView.setVisibility(View.VISIBLE);
        } else {
            fillScreenManager.showFromTop(binding.pdfToolBar, 200);
            binding.inkCtrlView.setVisibility(View.GONE);
            fillScreenManager.removeToolView(binding.inkCtrlView);
            fillScreenManager.showFromBottom(binding.annotationToolBar, 200);
        }
    }

    public void fillScreenChange() {
        fillScreenManager.fillScreenChange(binding.flTool.getVisibility() == View.VISIBLE);

    }

}
