/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.annotation.pdfproperties.pdfsignature;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.SeekBar;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatButton;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.cardview.widget.CardView;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions;
import com.compdfkit.tools.R;
import com.compdfkit.tools.annotation.pdfproperties.pdfsignature.data.CSignatureDatas;
import com.compdfkit.tools.common.utils.activitycontracts.CImageResultContracts;
import com.compdfkit.tools.common.utils.dialog.CImportImageDialogFragment;
import com.compdfkit.tools.common.utils.image.CBitmapUtil;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.view.colorpicker.ColorPickerDialogFragment;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.CToolBar;
import com.compdfkit.tools.common.views.pdfproperties.colorlist.ColorListView;
import com.compdfkit.tools.common.views.pdfproperties.writing.CKMHandWritingHelper;
import com.compdfkit.tools.common.views.pdfproperties.writing.CPDFWritingView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;


public class CAddSignatureActivity extends AppCompatActivity implements View.OnClickListener {

    private AppCompatImageView ivAddDrawSignature;

    private AppCompatImageView ivAddTextSignature;

    private AppCompatImageView ivAddPicSignature;

    private CToolBar cToolBar;

    private CPDFWritingView writingView;

    private AppCompatEditText editText;

    private CardView cardViewImport;

    private AppCompatImageView ivImport;

    private ColorListView colorListView;

    private AppCompatButton btnSave;

    private FloatingActionButton btnClean;

    private FloatingActionButton fabAddPic;

    private ConstraintLayout clThickness;

    private SeekBar sbThickness;

    private Uri imageUri;

    private ActivityResultLauncher<CImageResultContracts.RequestType> imageLauncher = registerForActivityResult(new CImageResultContracts(), result -> {
        if (result != null) {
            imageUri = result;
            Glide.with(this)
                    .load(result)
                    .transition(DrawableTransitionOptions.withCrossFade())
                    .into(ivImport);
            switchTab(2);
        }
    });

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.tools_properties_signature_style_add_activity);
        writingView = findViewById(R.id.writing_view);
        editText = findViewById(R.id.et_text);
        cardViewImport = findViewById(R.id.card_import_image);
        fabAddPic = findViewById(R.id.fab_add_pic);
        ivImport = findViewById(R.id.iv_image);
        ivAddDrawSignature = findViewById(R.id.iv_add_draw_signature);
        ivAddTextSignature = findViewById(R.id.iv_add_text_signature);
        ivAddPicSignature = findViewById(R.id.iv_add_pic_signature);
        cToolBar = findViewById(R.id.tool_bar);
        colorListView = findViewById(R.id.color_list_view);
        btnSave = findViewById(R.id.btn_save);
        btnClean = findViewById(R.id.btn_clean);
        clThickness = findViewById(R.id.cl_thickness);
        sbThickness = findViewById(R.id.seek_bar_thickness);
        initView();
        initListener();
        switchTab(0);
    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.iv_add_draw_signature) {
            switchTab(0);
        } else if (v.getId() == R.id.iv_add_text_signature) {
            switchTab(1);
        } else if (v.getId() == R.id.iv_add_pic_signature || v.getId() == R.id.fab_add_pic) {
            CViewUtils.hideKeyboard(editText);
            CImportImageDialogFragment importImageDialogFragment = CImportImageDialogFragment.newInstance();
            importImageDialogFragment.setPhotoAlbumClickListener(v1 -> {
                imageLauncher.launch(CImageResultContracts.RequestType.PHOTO_ALBUM);
                importImageDialogFragment.dismiss();
            });
            importImageDialogFragment.setCameraClickListener(v1 -> {
                imageLauncher.launch(CImageResultContracts.RequestType.CAMERA);
                importImageDialogFragment.dismiss();
            });
            importImageDialogFragment.show(getSupportFragmentManager(), "importImage");
        } else if (v.getId() == R.id.iv_tool_bar_back) {
            onBackPressed();
        } else if (v.getId() == R.id.btn_save) {
            saveBitmap();
        } else if (v.getId() == R.id.btn_clean) {
            cleanSignature();
        }
    }

    private void initListener() {
        fabAddPic.setOnClickListener(this);
        cToolBar.setBackBtnClickListener(this);
        ivAddDrawSignature.setOnClickListener(this);
        ivAddTextSignature.setOnClickListener(this);
        ivAddPicSignature.setOnClickListener(this);
        btnClean.setOnClickListener(this);
        btnSave.setOnClickListener(this);
        colorListView.setOnColorSelectListener(this::setSignatureColor);
        colorListView.setColorPickerClickListener(() -> {

            ColorPickerDialogFragment colorPickerDialogFragment = ColorPickerDialogFragment.newInstance(getSignatureColor());
            colorPickerDialogFragment.setColorChangeListener(color -> setSignatureColor(color));
            colorPickerDialogFragment.show(getSupportFragmentManager(), "colorPickerDialogFragment");
        });
        sbThickness.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                writingView.setPenWidth(Math.max(0, progress));
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    private void initView() {
        writingView.setDrawEffect(CKMHandWritingHelper.DrawEffect.NORMAL);
        writingView.setPenWidth(20);
        writingView.setColor(Color.BLACK);
    }

    private void switchTab(int selectPosition) {
        switch (selectPosition) {
            case 0:
                CViewUtils.hideKeyboard(editText);
                ivAddDrawSignature.setSelected(true);
                ivAddTextSignature.setSelected(false);
                ivAddPicSignature.setSelected(false);
                writingView.setVisibility(View.VISIBLE);
                editText.setVisibility(View.GONE);
                cardViewImport.setVisibility(View.GONE);
                colorListView.setVisibility(View.VISIBLE);
                clThickness.setVisibility(View.VISIBLE);
                colorListView.setSelectColor(getSignatureColor());
                fabAddPic.setVisibility(View.GONE);
                btnClean.setVisibility(View.VISIBLE);
                break;
            case 1:
                CViewUtils.showKeyboard(editText);
                ivAddDrawSignature.setSelected(false);
                ivAddTextSignature.setSelected(true);
                ivAddPicSignature.setSelected(false);
                writingView.setVisibility(View.GONE);
                editText.setVisibility(View.VISIBLE);
                cardViewImport.setVisibility(View.GONE);
                colorListView.setVisibility(View.VISIBLE);
                clThickness.setVisibility(View.GONE);
                colorListView.setSelectColor(getSignatureColor());
                fabAddPic.setVisibility(View.GONE);
                btnClean.setVisibility(View.VISIBLE);
                break;
            default:
                CViewUtils.hideKeyboard(editText);
                ivAddDrawSignature.setSelected(false);
                ivAddTextSignature.setSelected(false);
                ivAddPicSignature.setSelected(true);
                writingView.setVisibility(View.GONE);
                editText.setVisibility(View.GONE);
                clThickness.setVisibility(View.GONE);
                cardViewImport.setVisibility(View.VISIBLE);
                colorListView.setVisibility(View.INVISIBLE);
                fabAddPic.setVisibility(View.VISIBLE);
                btnClean.setVisibility(View.GONE);
                break;
        }
    }

    private void cleanSignature() {
        if (ivAddDrawSignature.isSelected()) {
            writingView.clean();
        } else if (ivAddTextSignature.isSelected()) {
            editText.setText("");
        } else {
            ivImport.setImageBitmap(null);
            imageUri = null;
        }
    }

    private void setSignatureColor(int color) {
        if (ivAddDrawSignature.isSelected()) {
            writingView.setColor(color);
        } else if (ivAddTextSignature.isSelected()) {
            editText.setTextColor(color);
        } else {

        }
    }

    private int getSignatureColor() {
        int color = Color.BLACK;
        if (ivAddDrawSignature.isSelected()) {
            color = writingView.getColor();
        } else if (ivAddTextSignature.isSelected()) {
            color = editText.getCurrentTextColor();
        }
        return color;
    }

    private void saveBitmap() {
        CThreadPoolUtils.getInstance().execute(() -> {
            String savePath = null;
            if (ivAddDrawSignature.isSelected()) {
                Bitmap resultBitmap = CBitmapUtil.cropTransparent(writingView.getBitmap());
                savePath = CSignatureDatas.saveSignatureBitmap(this, resultBitmap);
            } else if (ivAddTextSignature.isSelected()) {
                if (editText.getText() != null && editText.getText().length() > 0) {
                    editText.setCursorVisible(false);
                    editText.buildDrawingCache();
                    Bitmap bitmap = Bitmap.createBitmap(editText.getDrawingCache());
                    Bitmap resultBitmap = CBitmapUtil.cropTransparent(bitmap);
                    savePath = CSignatureDatas.saveSignatureBitmap(this, resultBitmap);
                }
            } else if (ivAddPicSignature.isSelected()) {
                if (imageUri != null) {
                    savePath = CSignatureDatas.saveSignatureImage(this, imageUri);
                }
            }
            if (savePath != null) {
                Intent intent = new Intent();
                intent.putExtra("file_path", savePath);
                setResult(Activity.RESULT_OK, intent);
                onBackPressed();
            }
        });
    }

}
