package com.compdfkit.tools.common.utils;

import android.content.ActivityNotFoundException;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.ParcelFileDescriptor;
import android.util.Log;

import androidx.core.content.FileProvider;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

public class CFileUtils {

    public static final String ROOT_FOLDER = "compdfkit/";

    public static final String CACHE_FOLDER = "compdfkit/temp";

    public static final String SIGNATURE_FOLDER = "compdfkit/annot/signature";

    public static final String IMAGE_STAMP_FOLDER = "compdfkit/annot/stamp/image";

    public static final String TEXT_STAMP_FOLDER = "compdfkit/annot/stamp/text";

    public static final String SOUND_FOLDER = "compdfkit/annot/sound";

    public static final String EXPORT_FOLDER = "compdfkit/edit/export";


    public static void copyFileFromAssets(Context context,
                                          String assetName,
                                          String savePath,
                                          String saveName,
                                          final boolean overwriteExisting) {
        //if save path folder not exists, create directory
        File dir = new File(savePath);
        if (!dir.exists()) {
            if (!dir.mkdir()) {
                Log.d("FileUtils", "mkdir error: " + savePath);
                return;
            }
        }

        // 拷贝文件
        String filename = savePath + "/" + saveName;
        File file = new File(filename);
        if (!file.exists() || overwriteExisting) {
            try {
                InputStream inStream = context.getAssets().open(assetName);
                FileOutputStream fileOutputStream = new FileOutputStream(filename);
                int byteread;
                byte[] buffer = new byte[1024];
                while ((byteread = inStream.read(buffer)) != -1) {
                    fileOutputStream.write(buffer, 0, byteread);
                }
                fileOutputStream.flush();
                inStream.close();
                fileOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            Log.d("FileUtils", "[copyFileFromAssets] copy asset file: " + assetName + " to : " + filename);
        } else {
            Log.d("FileUtils", "[copyFileFromAssets] file is exist: " + filename);
        }
    }


    public static void createFile(File file, boolean isFile) {// 创建文件
        try {
            if (!file.exists()) {
                // 如果文件不存在
                if (!file.getParentFile().exists()) {
                    // 如果文件父目录不存在
                    createFile(file.getParentFile(), false);
                } else {
                    // 存在文件父目录
                    if (isFile) {
                        // 创建文件
                        file.createNewFile();// 创建新文件
                    } else {
                        file.mkdir();// 创建目录
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static String copyImageToInternalDirectory(
            Context context,
            Uri imageUri,
            String dir,
            String fileName
    ) {
        try {
            File file = new File(dir, fileName);
            file.getParentFile().mkdirs();

            ContentResolver cr = context.getContentResolver();
            ParcelFileDescriptor fd = cr.openFileDescriptor(imageUri, "r");
            FileInputStream ist = new FileInputStream(fd.getFileDescriptor());
            FileOutputStream outputStream = new FileOutputStream(file.getAbsolutePath());
            if (writeFile(ist, outputStream)) {
                return file.getAbsolutePath();
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    public static boolean writeFileFromString(final File file,
                                              final String content,
                                              final boolean append) {
        if (file == null || content == null) {
            return false;
        }
        if (file.exists()) {
            return false;
        }
        if (file.isDirectory()) {
            return false;
        }
        BufferedWriter bw = null;
        try {
            file.createNewFile();
            bw = new BufferedWriter(new FileWriter(file, append));
            bw.write(content);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            try {
                if (bw != null) {
                    bw.close();
                }
            } catch (IOException e) {
            }
        }
    }

    public static String readFile2String(final File file, final String charsetName) {
        if (file == null || !file.exists()) {
            return null;
        }
        BufferedReader reader = null;
        try {
            StringBuilder sb = new StringBuilder();
            if (isSpace(charsetName)) {
                reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
            } else {
                reader = new BufferedReader(
                        new InputStreamReader(new FileInputStream(file), charsetName)
                );
            }
            String line;
            if ((line = reader.readLine()) != null) {
                sb.append(line);
                while ((line = reader.readLine()) != null) {
                    sb.append(System.getProperty("line.separator")).append(line);
                }
            }
            return sb.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException e) {
            }
        }
    }

    private static boolean isSpace(final String s) {
        if (s == null) {
            return true;
        }
        for (int i = 0, len = s.length(); i < len; ++i) {
            if (!Character.isWhitespace(s.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    public static Boolean writeFile(InputStream fileInputStream, OutputStream fileOutputStream) {

        try {

            byte[] data = new byte[2048];
            int length = -1;
            while ((length = fileInputStream.read(data)) != -1) {
                fileOutputStream.write(data, 0, length);
            }
            fileOutputStream.flush();
            return true;
        } catch (Exception e) {
            throw new RuntimeException("FileNotFoundException occurred. ", e);
        } finally {
            try {
                fileInputStream.close();
                fileOutputStream.close();
            } catch (IOException e) {
            }
        }
    }

    public static Uri getUriBySystem(Context context, File file) {
        try {
            if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.M) {
                return Uri.fromFile(file);
            } else {
                return FileProvider.getUriForFile(context, context.getPackageName() + ".fileprovider", file);
            }
        } catch (Exception e) {
            return null;
        }
    }

    public static void shareFile(Context context, String title, String type, File file) {
        try {
            Intent intent = new Intent(Intent.ACTION_SEND);
            intent.setType(type);
            intent.putExtra(Intent.EXTRA_SUBJECT, title);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.putExtra(Intent.EXTRA_STREAM, getUriBySystem(context, file));
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            context.startActivity(Intent.createChooser(intent, title));
        } catch (ActivityNotFoundException e) {
            e.printStackTrace();
        }
    }

    public static void shareFile(Context context, String title, String type, Uri uri) {
        try {
            Intent intent = new Intent(Intent.ACTION_SEND);
            intent.setType(type);
            intent.putExtra(Intent.EXTRA_SUBJECT, title);
            intent.putExtra(Intent.EXTRA_STREAM, uri);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            context.startActivity(Intent.createChooser(intent, title));
        } catch (ActivityNotFoundException e) {
            e.printStackTrace();
        }
    }

    public static Intent getContentIntent() {
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        intent.setType("application/pdf");
        intent.addFlags(Intent.FLAG_GRANT_PERSISTABLE_URI_PERMISSION);
        intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        return intent;
    }

    public static void takeUriPermission(Context context, Uri uri){
        try{
            context.getContentResolver().takePersistableUriPermission(uri, Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
        }catch (Exception e){

        }
    }
}
