package com.compdfkit.tools.common.utils.date;


import android.text.TextUtils;

import java.text.SimpleDateFormat;
import java.util.Date;

public class CDateUtil {

    public static final String NORMAL_DATE_FORMAT = "yyyy_MM_dd_HH_mm_ss";

    /**
     * Transform a PDF date string in the format D:20230130060237+00'00' to a human-readable yyyy-MM-dd HH:mm:ss date string.
     *
     * @param inputDate the PDF date string to be transformed
     * @return the transformed date string in yyyy-MM-dd HH:mm:ss format, or an empty string if the input string is null or too short
     */
    public static String transformPDFDate(String inputDate) {
        try{
            if (TextUtils.isEmpty(inputDate) || inputDate.length() < 16) {
                return "";
            }
            String year = inputDate.substring(2, 6);
            String month = inputDate.substring(6, 8);
            String day = inputDate.substring(8, 10);
            String hour = inputDate.substring(10, 12);
            String minute = inputDate.substring(12, 14);
            String second = inputDate.substring(14, 16);
            return year + "-" + month + "-" + day + " " + hour + ":" + minute + ":" + second;
        }catch (Exception e){
            return inputDate;
        }
    }

    public static String getDataTime(String format) {
        SimpleDateFormat df = new SimpleDateFormat(format);
        return df.format(new Date());
    }

}
