package com.compdfkit.tools.common.utils.window;


import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RadioButton;
import android.widget.RadioGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.dialog.CDialogFragmentUtil;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;

public class CModeSwitchDialogFragment extends BottomSheetDialogFragment implements RadioGroup.OnCheckedChangeListener {

    private OnPreviewModeChangeListener changeListener;

    private AppCompatImageView ivClose;

    private AppCompatTextView tvTitle;

    private RadioGroup radioGroup;

    private RadioButton rbViewer;

    private RadioButton rbAnnotation;

    private RadioButton rbPDFEdit;

    private RadioButton rbForm;

    private RadioButton rbPageEdit;

    private LinkedHashSet<CPreviewMode> previewModes = new LinkedHashSet<>();

    private CPreviewMode selectMode;

    public static CModeSwitchDialogFragment newInstance(LinkedHashSet<CPreviewMode> modes, CPreviewMode selectMode) {
        Bundle args = new Bundle();
        CModeSwitchDialogFragment fragment = new CModeSwitchDialogFragment();
        fragment.setArguments(args);
        fragment.setModes(modes);
        fragment.setSelectMode(selectMode);
        return fragment;
    }

    public void setModes(LinkedHashSet<CPreviewMode> modes){
        previewModes = modes;
    }

    public void setSelectMode(CPreviewMode selectMode){
        this.selectMode = selectMode;
    }

    @Override
    public void onStart() {
        super.onStart();
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        CDialogFragmentUtil.setDimAmount(getDialog(), 0.2F);
        behavior.setSkipCollapsed(true);
        behavior.setState(BottomSheetBehavior.STATE_EXPANDED);
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(STYLE_NORMAL, R.style.Tools_Base_Theme_BasicBottomSheetDialogStyle_TopCorners);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_pdf_mode_switch_dialog_fragment, container,false);
        ivClose = rootView.findViewById(R.id.iv_tool_bar_close);
        tvTitle = rootView.findViewById(R.id.tv_tool_bar_title);
        radioGroup = rootView.findViewById(R.id.radio_group_mode);
        rbViewer = rootView.findViewById(R.id.r_btn_viewer_mode);
        rbAnnotation = rootView.findViewById(R.id.r_btn_annotation_mode);
        rbPDFEdit = rootView.findViewById(R.id.r_btn_pdf_edit_mode);
        rbForm = rootView.findViewById(R.id.r_btn_form_mode);
        rbPageEdit = rootView.findViewById(R.id.r_btn_page_edit_mode);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (savedInstanceState != null){
            if (selectMode == null && savedInstanceState.containsKey("select_mode")) {
                selectMode = CPreviewMode.valueOf(savedInstanceState.getString("select_mode"));
            }
            if (previewModes.size() <=0 && savedInstanceState.containsKey("modes")){
                try {
                    previewModes = (LinkedHashSet<CPreviewMode>) savedInstanceState.getSerializable("modes");
                }catch (Exception e){

                }
            }
        }
        tvTitle.setText(R.string.tools_pdf_mode_title);
        ivClose.setOnClickListener(v -> {
            dismiss();
        });
        switch (selectMode){
            case Viewer:
                rbViewer.setChecked(true);
                break;
            case Annotation:
                rbAnnotation.setChecked(true);
                break;
            case Edit:
                rbPDFEdit.setChecked(true);
                break;
            case Form:
                rbForm.setChecked(true);
                break;
            case PageEdit:
                rbPageEdit.setChecked(true);
                break;
            default:break;
        }
        modifyPopupMenuStatus();
        radioGroup.setOnCheckedChangeListener(this);
    }

    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
        if (checkedId == R.id.r_btn_viewer_mode) {
            changeMode(CPreviewMode.Viewer);
        } else if (checkedId == R.id.r_btn_annotation_mode) {
            changeMode(CPreviewMode.Annotation);
        } else if (checkedId == R.id.r_btn_pdf_edit_mode) {
            changeMode(CPreviewMode.Edit);
        } else if (checkedId == R.id.r_btn_page_edit_mode) {
            changeMode(CPreviewMode.PageEdit);
        } else if (checkedId == R.id.r_btn_form_mode) {
            changeMode(CPreviewMode.Form);
        }
        dismiss();
    }

    private void modifyPopupMenuStatus() {
        if (rbViewer != null) {
            rbViewer.setVisibility(previewModes.contains(CPreviewMode.Viewer) ? View.VISIBLE : View.GONE);
        }
        if (rbAnnotation != null) {
            rbAnnotation.setVisibility(previewModes.contains(CPreviewMode.Annotation) ? View.VISIBLE : View.GONE);
        }
        if (rbPDFEdit != null) {
            rbPDFEdit.setVisibility(previewModes.contains(CPreviewMode.Edit) ? View.VISIBLE : View.GONE);
        }
        if (rbForm != null) {
            rbForm.setVisibility(previewModes.contains(CPreviewMode.Form) ? View.VISIBLE : View.GONE);
        }
        if (rbPageEdit != null) {
            rbPageEdit.setVisibility(previewModes.contains(CPreviewMode.PageEdit) ? View.VISIBLE : View.GONE);
        }
    }


    private void changeMode(CPreviewMode mode) {
        selectMode = mode;
        if (changeListener != null) {
            changeListener.change(selectMode);
        }
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState) {
        outState.putString("select_mode", selectMode.name());
        outState.putSerializable("modes", previewModes);
        super.onSaveInstanceState(outState);
    }

    public void setSwitchModeListener(OnPreviewModeChangeListener changeListener) {
        this.changeListener = changeListener;
    }

    public interface OnPreviewModeChangeListener {
        void change(CPreviewMode mode);
    }
}
