package com.compdfkit.tools.common.views.pdfpageedit;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RadioButton;
import android.widget.RadioGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.dialog.CDialogFragmentUtil;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.viewutils.CDimensUtils;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.CPageEditBar;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

public class CInsertBlankPageDialogFragment extends BottomSheetDialogFragment implements View.OnClickListener {
    private CPageEditBar toolBar;
    private CPDFViewCtrl pdfView;
    private ConstraintLayout clPagesize;
    private AppCompatTextView tvCurPagesize;
    private RadioGroup rgPagesize;
    private AppCompatImageView ivDirectionV;
    private AppCompatImageView ivDirectionH;
    private AppCompatImageView ivRightArrow;
    private RadioButton rbHomePage;
    private RadioButton rbLastPage;
    private RadioButton rbToPageLocationBefore;
    private RadioButton rbToPageLocationAfter;
    private AppCompatEditText etInputPageIndex;

    private CPageEditBar.OnEditDoneCallback onEditDoneCallback;

    private int pageIndex;
    public static CInsertBlankPageDialogFragment newInstance(){
        return new CInsertBlankPageDialogFragment();
    }

    public void initWithPDFView(CPDFViewCtrl pdfView){
        this.pdfView = pdfView;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(STYLE_NORMAL, R.style.Tools_Base_Theme_BasicBottomSheetDialogStyle);
    }

    @Override
    public void onStart() {
        super.onStart();
        if (!CViewUtils.isLandScape(getContext())){
            CDialogFragmentUtil.setDimAmount(getDialog(), 0F);
        }
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        CDialogFragmentUtil.setBottomSheetDialogFragmentFullScreen(getDialog(), behavior);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_pageedit_insert_blank_page_dialog_fragment, container, false);
        toolBar = rootView.findViewById(R.id.page_edit_tool_bar);
        ivDirectionV = rootView.findViewById(R.id.iv_tool_insert_page_verticel);
        ivDirectionH = rootView.findViewById(R.id.iv_tool_insert_page_horizontal);
        ivRightArrow = rootView.findViewById(R.id.iv_tool_insert_page_right);
        tvCurPagesize = rootView.findViewById(R.id.iv_tool_insert_page_cur_size);
        clPagesize = rootView.findViewById(R.id.cl_pagesize);
        rgPagesize = rootView.findViewById(R.id.rg_tools_edit_page_insertpage_pagesize);
        rbHomePage = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_homepage);
        rbLastPage = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_lastpage);
        rbToPageLocationBefore = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_before);
        rbToPageLocationAfter = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_after);
        etInputPageIndex = rootView.findViewById(R.id.et_tool_edit_page_enterpage);

        rbHomePage.setOnClickListener(this);
        rbLastPage.setOnClickListener(this);
        rbToPageLocationBefore.setOnClickListener(this);
        rbToPageLocationAfter.setOnClickListener(this);
        etInputPageIndex.setOnFocusChangeListener((view, b) -> {
            if (b) {
                if (rbHomePage.isChecked() || rbLastPage.isChecked()) {
                    rbToPageLocationBefore.setChecked(true);
                    CViewUtils.showKeyboard(etInputPageIndex);
                }
            } else {
                CViewUtils.hideKeyboard(etInputPageIndex);
            }
        });

        toolBar.setBackBtnClickListener(v -> dismiss());
        toolBar.setOnDoneClickCallback(() -> CThreadPoolUtils.getInstance().execute(() -> {
            insertBlankPage();
            dismiss();
            if (onEditDoneCallback != null) {
                onEditDoneCallback.onEditDone();
            }
        }));
        toolBar.setTitle(R.string.tools_page_edit_insert_page_blank);
        toolBar.showEditButton(false);
        toolBar.showSelectButton(false);
        toolBar.showDoneButton(true);
        ivDirectionV.setSelected(true);
        ivDirectionV.setOnClickListener(this);
        ivDirectionH.setOnClickListener(this);
        ivRightArrow.setOnClickListener(this);

        return rootView;
    }

    public int getInsertPageIndex() {
        return pageIndex;
    }

    private void insertBlankPage() {
        if (pdfView == null || pdfView.getCPdfReaderView() == null) {
            return;
        }
        CPDFDocument document = pdfView.getCPdfReaderView().getPDFDocument();
        if (document == null) {
            return;
        }
        String pageStr = etInputPageIndex.getText().toString();
        pageIndex = 1;
        if (rbHomePage.isChecked()) {
            pageIndex = 0;
        } else if (rbLastPage.isChecked()) {
            pageIndex = pdfView.getCPdfReaderView().getPDFDocument().getPageCount();
        } else {
            if (!TextUtils.isEmpty(pageStr)) {
                try {
                    pageIndex = Integer.parseInt(pageStr);
                } catch (Exception e) {

                }
            }
            if (pageIndex >= 1) {
                if (rbToPageLocationBefore.isChecked()) {
                    pageIndex -= 1;
                }
            }
        }

        String pageSize = "";
        for (int i = 0; i < rgPagesize.getChildCount(); i++) {
            if (((RadioButton)rgPagesize.getChildAt(i)).isChecked()) {
                pageSize = (String)rgPagesize.getChildAt(i).getTag();
                break;
            }
        }
        if (!TextUtils.isEmpty(pageSize)) {
            String[] size = pageSize.split("\\*");
            if (size != null && size.length == 2) {
                int width = Integer.parseInt(size[0]);
                int height = Integer.parseInt(size[1]);
                if (ivDirectionH.isSelected()) {
                    document.insertBlankPage(pageIndex, height, width);
                } else {
                    document.insertBlankPage(pageIndex, width, height);
                }
            }
        }
    }

    @SuppressLint("ResourceType")
    private void initPageSizeGroup() {
        String[] sizeArr = getContext().getResources().getStringArray(R.array.tools_page_edit_pagesize_name);
        String[] sizeValueArr = getContext().getResources().getStringArray(R.array.tools_page_edit_pagesize_value);
        if (sizeArr == null) {
            return;
        }
        for (int i = 0; i < sizeArr.length; i++) {
            RadioButton radio = (RadioButton)LayoutInflater.from(getContext()).inflate(R.layout.tools_pageedit_pagesize_item, null);
            RadioGroup.LayoutParams layoutParams =
                    new RadioGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, CDimensUtils.dp2px(getContext(), 50));
            radio.setLayoutParams(layoutParams);
            radio.setId(View.generateViewId());
            if (i == 1) {
                radio.setChecked(true);
                tvCurPagesize.setText(sizeArr[i]);
            }
            radio.setText(sizeArr[i]);
            radio.setTag(sizeValueArr[i]);
            radio.setOnClickListener(view -> {
                tvCurPagesize.setText(((RadioButton)view).getText());
            });
            rgPagesize.addView(radio,i);
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        initPageSizeGroup();
    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        if (id == R.id.iv_tool_insert_page_verticel) {
            boolean select = ivDirectionV.isSelected();
            if (!select) {
                ivDirectionV.setSelected(true);
                ivDirectionH.setSelected(false);
            }
        } else if (id == R.id.iv_tool_insert_page_horizontal) {
            boolean select = ivDirectionH.isSelected();
            if (!select) {
                ivDirectionH.setSelected(true);
                ivDirectionV.setSelected(false);
            }
        } else if (id == R.id.iv_tool_insert_page_right) {
            boolean select = ivRightArrow.isSelected();
            ivRightArrow.setSelected(!select);
            if (!select == true) {
                clPagesize.setVisibility(View.VISIBLE);
            } else {
                clPagesize.setVisibility(View.GONE);
            }
        } else if (id == R.id.rb_tools_edit_page_insert_location_homepage) {
            etInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_lastpage) {
            etInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_before) {
            if (!etInputPageIndex.isFocused()) {
                etInputPageIndex.requestFocus();
                CViewUtils.showKeyboard(etInputPageIndex);
            }
        } else if (id == R.id.rb_tools_edit_page_insert_location_after) {
            if (!etInputPageIndex.isFocused()) {
                etInputPageIndex.requestFocus();
                CViewUtils.showKeyboard(etInputPageIndex);
            }
        }
    }

    public void setOnEditDoneCallback(CPageEditBar.OnEditDoneCallback callback) {
        onEditDoneCallback = callback;
    }
}
