package com.compdfkit.tools.common.views.pdfpageedit;

import static com.compdfkit.core.document.CPDFDocument.PDFDocumentError.PDFDocumentErrorPassword;
import static com.compdfkit.core.document.CPDFDocument.PDFDocumentError.PDFDocumentErrorSuccess;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RadioButton;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.fragment.app.FragmentActivity;

import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.dialog.CDialogFragmentUtil;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.CPageEditBar;
import com.compdfkit.tools.common.views.CVerifyPasswordDialogFragment;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CInsertPdfPageDialogFragment extends BottomSheetDialogFragment implements View.OnClickListener {
    private CPageEditBar toolBar;
    private CPDFViewCtrl pdfView;
    private AppCompatTextView tvFilename;
    private AppCompatImageView ivFileNameRight;
    private RadioButton rbFromPageLocationAll;
    private RadioButton rbFromPageLocationOdd;
    private RadioButton rbFromPageLocationEven;
    private RadioButton rbFromPageLocationSpecify;
    private AppCompatEditText etFromInputPageIndex;

    private RadioButton rbToPageLocationHome;
    private RadioButton rbToPageLocationLast;
    private RadioButton rbToPageLocationBefore;
    private RadioButton rbToPageLocationAfter;
    private AppCompatEditText etToInputPageIndex;

    private CPageEditBar.OnEditDoneCallback onEditDoneCallback;
    private Uri selectUri = null;
    private CPDFDocument selectDocument = null;

    private int[] insertPages = null;
    private final String selectPageRegex = "^(\\d+)(-\\d+)?([,，](\\d+)(-\\d+)?)*$";
    private ActivityResultLauncher<Intent> selectDocumentLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getData() != null && result.getData().getData() != null) {
            selectUri = result.getData().getData();
            String path = selectUri.getPath();
            String[] arr = path.split("/");
            if (arr != null && arr.length > 0) {
                tvFilename.setText(arr[arr.length - 1]);
            }
            CFileUtils.takeUriPermission(getContext(), selectUri);
            selectDocument = new CPDFDocument(getContext());
            CPDFDocument.PDFDocumentError pdfDocumentError = selectDocument.open(selectUri);
            if (pdfDocumentError == PDFDocumentErrorSuccess) {

            } else if (pdfDocumentError == PDFDocumentErrorPassword) {
                CVerifyPasswordDialogFragment verifyPasswordDialogFragment;
                verifyPasswordDialogFragment = CVerifyPasswordDialogFragment.newInstance(selectDocument, selectUri);
                verifyPasswordDialogFragment.setVerifyCompleteListener(document -> {

                });
                if (getContext() instanceof FragmentActivity){
                    verifyPasswordDialogFragment.show(((FragmentActivity) getContext()).getSupportFragmentManager(), "verifyPwdDialog");
                }
            }
        }
    });

    public static CInsertPdfPageDialogFragment newInstance(){
        return new CInsertPdfPageDialogFragment();
    }

    public void initWithPDFView(CPDFViewCtrl pdfView){
        this.pdfView = pdfView;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(STYLE_NORMAL, R.style.Tools_Base_Theme_BasicBottomSheetDialogStyle);
    }

    @Override
    public void onStart() {
        super.onStart();
        if (!CViewUtils.isLandScape(getContext())){
            CDialogFragmentUtil.setDimAmount(getDialog(), 0F);
        }
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        CDialogFragmentUtil.setBottomSheetDialogFragmentFullScreen(getDialog(), behavior);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_pageedit_insert_pdf_page_dialog_fragment, container, false);
        toolBar = rootView.findViewById(R.id.page_edit_tool_bar);
        tvFilename = rootView.findViewById(R.id.iv_tool_insert_page_filename);

        ivFileNameRight = rootView.findViewById(R.id.iv_tool_insert_page_right);
        rbFromPageLocationAll = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_from_all);
        rbFromPageLocationOdd = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_from_odd);
        rbFromPageLocationEven = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_from_even);
        rbFromPageLocationSpecify = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_from_specify);
        etFromInputPageIndex = rootView.findViewById(R.id.et_tool_edit_page_from_enterpage);

        rbToPageLocationHome = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_homepage);
        rbToPageLocationLast = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_lastpage);
        rbToPageLocationBefore = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_before);
        rbToPageLocationAfter = rootView.findViewById(R.id.rb_tools_edit_page_insert_location_after);
        etToInputPageIndex = rootView.findViewById(R.id.et_tool_edit_page_enterpage);

        rbFromPageLocationAll.setOnClickListener(this);
        rbFromPageLocationOdd.setOnClickListener(this);
        rbFromPageLocationEven.setOnClickListener(this);
        rbFromPageLocationSpecify.setOnClickListener(this);

        rbToPageLocationHome.setOnClickListener(this);
        rbToPageLocationLast.setOnClickListener(this);
        rbToPageLocationBefore.setOnClickListener(this);
        rbToPageLocationAfter.setOnClickListener(this);

        etFromInputPageIndex.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if (b) {
                    if (!rbFromPageLocationSpecify.isChecked()) {
                        rbFromPageLocationSpecify.setChecked(true);
                        CViewUtils.showKeyboard(etFromInputPageIndex);
                    }
                } else {
                    CViewUtils.hideKeyboard(etFromInputPageIndex);
                }
            }
        });

        etToInputPageIndex.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if (b) {
                    if (rbToPageLocationHome.isChecked() || rbToPageLocationLast.isChecked()) {
                        rbToPageLocationBefore.setChecked(true);
                        CViewUtils.showKeyboard(etToInputPageIndex);
                    }
                } else {
                    CViewUtils.hideKeyboard(etToInputPageIndex);
                }
            }
        });

        toolBar.setBackBtnClickListener(v -> dismiss());
        toolBar.setOnDoneClickCallback(() -> {
            if (selectUri == null) {
                Toast.makeText(getContext(), R.string.tools_page_edit_select_file_tips, Toast.LENGTH_LONG).show();
            } else {
                boolean format = true;
                if (rbFromPageLocationSpecify.isChecked()) {
                    Pattern p = Pattern.compile(selectPageRegex);
                    Matcher m = p.matcher(etFromInputPageIndex.getText().toString());
                    format = m.matches();
                }
                if (!format) {
                    Toast.makeText(getContext(), R.string.tools_page_edit_insert_page, Toast.LENGTH_SHORT).show();
                } else {
                    CThreadPoolUtils.getInstance().execute(() -> {
                        insertPdfPage();
                        dismiss();
                        if (onEditDoneCallback != null) {
                            onEditDoneCallback.onEditDone();
                        }
                    });
                }
            }
        });
        toolBar.setTitle(R.string.tools_page_edit_insert_page_pdf);
        toolBar.showEditButton(false);
        toolBar.showSelectButton(false);
        toolBar.showDoneButton(true);
        ivFileNameRight.setOnClickListener(this);
        return rootView;
    }

    private int[] getPageArr(String pageStr) {
        if (TextUtils.isEmpty(pageStr)) {
            return null;
        }
        List<Integer> pageList= new ArrayList<>();
        int[] pageNum = null;
        String[] dotSplit = pageStr.split(",|，");
        if (dotSplit != null && dotSplit.length > 0) {
            try {
                for (int i = 0; i < dotSplit.length; i++) {
                    String[] lineSplit = dotSplit[i].split("-");
                    if (lineSplit == null) {
                        continue;
                    }
                    if (lineSplit.length == 1 ) {
                        int page = -1;
                        try {
                            page = Integer.parseInt(lineSplit[0]);
                            pageList.add(page);
                        } catch (Exception e) {

                        }
                    } else if (lineSplit.length == 2) {
                        int pageStart = -1;
                        int pageEnd = -1;
                        try {
                            pageStart = Integer.parseInt(lineSplit[0]);
                            pageEnd = Integer.parseInt(lineSplit[1]);
                            for (int j = pageStart; j <= pageEnd; j++) {
                                pageList.add(j);
                            }
                        } catch (Exception e) {

                        }
                    }
                }
            } catch (Exception e) {

            }
            if (pageList.size() > 0) {
                pageNum = new int[pageList.size()];
                for (int i = 0; i < pageList.size(); i++) {
                    pageNum[i] = pageList.get(i) - 1;
                }
            }
        }
        return pageNum;
    }

    private void insertPdfPage() {
        if (pdfView == null || pdfView.getCPdfReaderView() == null) {
            return;
        }
        CPDFDocument document = pdfView.getCPdfReaderView().getPDFDocument();
        if (document == null) {
            return;
        }
        if (selectDocument == null) {
            return;
        }
        int[] pageNums = null;
        if (rbFromPageLocationAll.isChecked()) {
            pageNums = new int[selectDocument.getPageCount()];
            for (int i = 0; i < pageNums.length; i++) {
                pageNums[i] = i;
            }
        } else if (rbFromPageLocationOdd.isChecked()) {
            pageNums = new int[(selectDocument.getPageCount() + 1) / 2];
            for (int i = 0, j = 0; i < selectDocument.getPageCount(); i += 2) {
                pageNums[j++] = i;
            }
        } else if (rbFromPageLocationEven.isChecked()) {
            pageNums = new int[selectDocument.getPageCount() / 2];
            for (int i = 1, j = 0; i < selectDocument.getPageCount(); i += 2) {
                pageNums[j++] = i;
            }
        } else {
            pageNums = getPageArr(etFromInputPageIndex.getText().toString());
        }

        int insertPos = 1;
        if (rbToPageLocationHome.isChecked()) {
            insertPos = 0;
        } else if (rbToPageLocationLast.isChecked()) {
            insertPos = document.getPageCount();
        } else {
            String toStr = etToInputPageIndex.getText().toString();
            try {
                insertPos = Integer.parseInt(toStr);
            } catch (Exception e) {

            }
            if (rbToPageLocationBefore.isChecked()) {
                insertPos -= 1;
            }
        }
        if (pageNums != null && pageNums.length > 0 && insertPos >= 0) {
            document.importPages(selectDocument, pageNums, insertPos);
            insertPages = new int[pageNums.length];
            for (int i = insertPos; i <= pageNums.length + insertPos - 1; i++) {
                insertPages[i - insertPos] = i;
            }
        }
    }

    public int[] getInsertPagesArr() {
        return insertPages;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {

    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        if (id == R.id.iv_tool_insert_page_right) {
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
        } else if (id == R.id.rb_tools_edit_page_insert_location_from_all) {
            etFromInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_from_odd) {
            etFromInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_from_even) {
            etFromInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_from_specify) {
            if (!etFromInputPageIndex.isFocused()) {
                etFromInputPageIndex.requestFocus();
                CViewUtils.showKeyboard(etFromInputPageIndex);
            }
        } else if (id == R.id.rb_tools_edit_page_insert_location_homepage) {
            etToInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_lastpage) {
            etToInputPageIndex.clearFocus();
        } else if (id == R.id.rb_tools_edit_page_insert_location_before) {
            if (!etToInputPageIndex.isFocused()) {
                etToInputPageIndex.requestFocus();
                CViewUtils.showKeyboard(etToInputPageIndex);
            }
        } else if (id == R.id.rb_tools_edit_page_insert_location_after) {
            if (!etToInputPageIndex.isFocused()) {
                etToInputPageIndex.requestFocus();
                CViewUtils.showKeyboard(etToInputPageIndex);
            }
        }
    }

    public void setOnEditDoneCallback(CPageEditBar.OnEditDoneCallback callback) {
        onEditDoneCallback = callback;
    }
}
