package com.compdfkit.tools.common.views.pdfpageedit.adapter;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.Lifecycle;
import androidx.viewpager2.adapter.FragmentStateAdapter;

import com.compdfkit.tools.common.interfaces.COnSetPDFDisplayPageIndexListener;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaEmptyFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.tools.viewer.pdfthumbnail.CPDFEditThumbnailFragment;

import java.util.ArrayList;

public class CPageEditViewPagerAdapter extends FragmentStateAdapter {

    /**
     * The PDF reader view object
     */
    private CPDFViewCtrl pdfView;

    private ArrayList<CPDFBotaFragmentTabs> tabs;

    /**
     * Listener for setting the display page index of the PDF
     */
    private COnSetPDFDisplayPageIndexListener pdfDisplayPageIndexListener;

    public CPageEditViewPagerAdapter(@NonNull FragmentManager fragmentManager, @NonNull Lifecycle lifecycle, CPDFViewCtrl pdfView, ArrayList<CPDFBotaFragmentTabs> tabs) {
        super(fragmentManager, lifecycle);
        this.pdfView = pdfView;
        this.tabs = tabs;
    }

    @NonNull
    @Override
    public Fragment createFragment(int position) {
        switch (tabs.get(position).getBotaType()) {
            case CPDFBOTA.THUMBNAIL:
                return editThumbnailFragment();
            default:
                return new CPDFBotaEmptyFragment();
        }
    }

    @Override
    public int getItemCount() {
        return tabs.size();
    }

    /**
     * Method that creates and returns a new thumbnail fragment
     * @return CPDFThumbnailFragment
     */
    private CPDFEditThumbnailFragment editThumbnailFragment() {
        CPDFEditThumbnailFragment fragment = CPDFEditThumbnailFragment.newInstance();
        fragment.setPDFDisplayPageIndexListener(pdfDisplayPageIndexListener);
        fragment.initWithPDFView(pdfView);
        return fragment;
    }

    /**
     * Method that sets the PDF display page index listener
     * @param displayPageIndexListener
     */
    public void setPDFDisplayPageIndexListener(COnSetPDFDisplayPageIndexListener displayPageIndexListener) {
        this.pdfDisplayPageIndexListener = displayPageIndexListener;
    }
}
