/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfproperties.writing;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.graphics.RectF;
import android.util.AttributeSet;

import androidx.annotation.Nullable;

import com.compdfkit.ui.utils.CPDFBitmapUtils;

public class CPDFWritingView extends CKmWritingView {

    private Canvas canvas;

    private Bitmap backgroundBitmap;

    private int penColor;

    public CPDFWritingView(Context context) {
        super(context);
    }

    public CPDFWritingView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public CPDFWritingView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @Override
    public void clean() {
        super.clean();
        if (backgroundBitmap != null && !backgroundBitmap.isRecycled()) {
            backgroundBitmap.recycle();
        }
        backgroundBitmap = null;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (handwritingHelper != null) {
            if (backgroundBitmap != null && !backgroundBitmap.isRecycled()) {
                canvas.drawBitmap(backgroundBitmap, (float) ((handwritingHelper.getCanvasWidth() - getWidth()) / 2),
                        (float) ((handwritingHelper.getCanvasHeight() - getHeight()) / 2), null);
            }
        }
        super.onDraw(canvas);
    }

    @Override
    public boolean isBlank() {
        if (backgroundBitmap != null && !backgroundBitmap.isRecycled()) {
            return false;
        } else {
            return super.isBlank();
        }
    }

    @Override
    public Bitmap getBitmap() {
        Bitmap handWritingBitmap = super.getBitmap();
        if (handWritingBitmap == null || handWritingBitmap.isRecycled()){
            return null;
        }
        Bitmap viewBitmap = Bitmap.createBitmap(getWidth(), getHeight(), Bitmap.Config.ARGB_8888);
        canvas = new Canvas(viewBitmap);
        if (handwritingHelper != null) {
            RectF area = handwritingHelper.getArea();
            if (backgroundBitmap != null && !backgroundBitmap.isRecycled()) {
                float bitmapWidth = getWidth();
                float bitmapHeight = getHeight();
                float canvasWidth = handwritingHelper.getCanvasWidth();
                float canvasHeight = handwritingHelper.getCanvasHeight();
                area.union(new RectF(((canvasWidth - bitmapWidth) / 2), ((canvasHeight - bitmapHeight) / 2), ((canvasWidth + bitmapWidth) / 2), ((canvasHeight + bitmapHeight) / 2)));
                canvas.drawBitmap(backgroundBitmap, ((canvasWidth - bitmapWidth) / 2), ((canvasHeight - bitmapHeight) / 2), null);
            }
            canvas.drawBitmap(handWritingBitmap, 0f, 0f, null);
            return CPDFBitmapUtils.cropBitmap(viewBitmap, new Rect((int) area.left, (int) area.top, (int) area.right, (int) area.bottom));
        } else {
            return null;
        }
    }


    @Override
    public void setColor(int color) {
        penColor = color;
        super.setColor(color);
        handwritingHelper.refreshSrcPointsColor(color);
        invalidate();
    }

    @Override
    public void setPenWidth(float penWidth) {
        super.setPenWidth(penWidth);
        handwritingHelper.refreshPenWidth(penWidth);
        invalidate();
    }

    public int getColor(){
        return penColor;
    }
}
