package com.compdfkit.tools.viewer.pdfsearch;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.dialog.CDialogFragmentUtil;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.CToolBar;
import com.compdfkit.tools.viewer.pdfsearch.adapter.CSearchPDFTextRecyclerviewAdapter;
import com.compdfkit.tools.viewer.pdfsearch.bean.CSearchTextInfo;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.List;

public class CSearchResultDialogFragment extends BottomSheetDialogFragment {
    private View mContentView;

    private CToolBar toolBar;

    private RecyclerView recyclerView;

    private ConstraintLayout clSearchResultEmpty;

    private List<CSearchTextInfo> searchTextInfos = null;

    private CSearchPDFTextRecyclerviewAdapter searchTextAdapter;

    private CSearchPDFTextRecyclerviewAdapter.OnClickSearchItemListener onClickSearchItemListener;
    //private OnClickBackListener onClickBackListener;
    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onStart() {
        super.onStart();
        if (!CViewUtils.isLandScape(getContext())){
            CDialogFragmentUtil.setDimAmount(getDialog(), 0F);
        }
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        CDialogFragmentUtil.setBottomSheetDialogFragmentFullScreen(getDialog(), behavior);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.tools_search_keywords_list_dialog_fragment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mContentView = view;
        initView();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    private void initView() {
        toolBar = mContentView.findViewById(R.id.id_search_head);
        recyclerView = mContentView.findViewById(R.id.id_reader_search_recyclerView);
        clSearchResultEmpty = mContentView.findViewById(R.id.cl_search_result_empty_view);
        AppCompatTextView tvResultInfo = mContentView.findViewById(R.id.tv_search_result);
        if (searchTextInfos != null) {
            tvResultInfo.setText(getContext().getString(R.string.tools_search_result_found, searchTextInfos.size()));
        }
        LinearLayoutManager layoutManager = new LinearLayoutManager(recyclerView.getContext());
        recyclerView.setLayoutManager(layoutManager);
        searchTextAdapter = new CSearchPDFTextRecyclerviewAdapter();
        recyclerView.setAdapter(searchTextAdapter);
        if (null != searchTextInfos && searchTextInfos.size() > 0) {
            searchTextAdapter.clearList();
            searchTextAdapter.addList(searchTextInfos);
            searchTextAdapter.notifyDataSetChanged();
        }else {
            clSearchResultEmpty.setVisibility(View.VISIBLE);
        }
        toolBar.setBackBtnClickListener(view -> {
            dismiss();
        });
        searchTextAdapter.setOnClickSearchItemListener(onClickSearchItemListener);
    }

    public void setSearchTextInfos(List<CSearchTextInfo> searchTextInfos) {
        this.searchTextInfos = searchTextInfos;
        if (null != searchTextAdapter) {
            searchTextAdapter.clearList();
            searchTextAdapter.addList(searchTextInfos);
            searchTextAdapter.notifyDataSetChanged();
        }
    }

    public void setOnClickSearchItemListener(CSearchPDFTextRecyclerviewAdapter.OnClickSearchItemListener listener) {
        onClickSearchItemListener = listener;
        if (null != searchTextAdapter) {
            searchTextAdapter.setOnClickSearchItemListener(listener);
        }
    }

//    public void setOnClickBackListener(OnClickBackListener listener) {
//        onClickBackListener = listener;
//    }
//
//    public interface OnClickBackListener {
//        public void onBack();
//    }
}
