package com.compdfkit.demo.pageedit;

import android.Manifest;
import android.content.Intent;
import android.graphics.Paint;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.compdfkit.demo.pageedit.databinding.ActivityMainBinding;
import com.compdfkit.tools.common.activity.CBasicActivity;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfpageedit.CPDFPageEditDialogFragment;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;

import java.util.ArrayList;

import pub.devrel.easypermissions.EasyPermissions;

public class MainActivity extends CBasicActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "Quick Start Guide V2.4.0.pdf";
    ActivityMainBinding binding;
    CPageEditSampleScreenManager screenManager = new CPageEditSampleScreenManager();

    private ActivityResultLauncher<Intent> selectDocumentLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getData() != null && result.getData().getData() != null) {
            Uri uri = result.getData().getData();
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfView.openPDF(uri);
            binding.pdfToolBar.selectMode(CPreviewMode.Viewer);
        }
    });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        initPDFView();
        initToolbarListener();
        initSearchBar();
    }

    private void setPreviewMode(CPreviewMode mode) {
        if (binding.pdfView.getCPdfReaderView() == null) {
            return;
        }
        binding.pdfToolBar.selectMode(mode);
        screenManager.changeWindowStatus(mode);
        binding.pdfView.setAllowAddAndEditAnnot(mode == CPreviewMode.Annotation);
        if (mode == CPreviewMode.Viewer) {

        } else {
            CPDFPageEditDialogFragment pageEditDialogFragment = CPDFPageEditDialogFragment.newInstance();
            pageEditDialogFragment.initWithPDFView(binding.pdfView);
            pageEditDialogFragment.setOnBackLisener(() -> {
                setPreviewMode(CPreviewMode.Viewer);
            });
            pageEditDialogFragment.show(getSupportFragmentManager(),"fff");
        }
    }

    private void initPDFView() {
        binding.pdfView.setAllowAddAndEditAnnot(false);
        resetContextMenu(binding.pdfView, CPreviewMode.Viewer);
        //Extract PDF files from the Android assets folder
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) -> {
            binding.pdfView.openPDF(filePath);
            setPreviewMode(CPreviewMode.Viewer);
        });
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            screenManager.fillScreenChange();
        });
    }

    private void initToolbarListener() {
        binding.pdfToolBar.addMode(CPreviewMode.PageEdit);
        binding.pdfToolBar.setSearchBtnClickListener(v -> {
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });
        binding.pdfToolBar.setBoTaBtnClickListener(v -> {
            //Show a fragment of the PDF thumbnail and a dialog fragment of the PDF outline list.
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs outlineTabs = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarks = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(outlineTabs);
            tabs.add(bookmarks);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });

        binding.pdfToolBar.setThumbnailBtnClickListener(v -> {
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs thumbnailTabs = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(thumbnailTabs);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });

        binding.pdfToolBar.setMoreBtnClickListener(v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (hasPermissions(STORAGE_PERMISSIONS)) {
                    selectDocument();
                } else {
                    EasyPermissions.requestPermissions(this, "request permission", REQUEST_EXTERNAL_PERMISSION, STORAGE_PERMISSIONS);
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });
        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            setPreviewMode(mode);
        });
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });

        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void selectDocument() {
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()){
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_EXTERNAL_PERMISSION) {
            if (EasyPermissions.hasPermissions(this, android.Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                selectDocument();
            }
        }
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed());
        }
    }
}